class
	CODE_TRANSFORMER

create
	make

feature {NONE}

	make (init_c_file, init_output_dir: STRING)
		require
			c_file_exists: (create {RAW_FILE}.make (init_c_file)).exists
			output_dir_exists: (create {DIRECTORY}.make (init_output_dir)).exists
		do
			c_file := init_c_file
			output_dir := init_output_dir
			steps :=
				<<
					create {SOURCE_LOADER},
					create {TREE_TRANSFORMER}
				>>
			current_step_index := 0
			create {LINKED_LIST [ PROCEDURE [ ANY, TUPLE [ INTEGER ] ] ]}progress_listener.make
		ensure
			c_file_assigned: c_file = init_c_file
			output_dir_assigned: output_dir = init_output_dir
			steps_initialized: steps /= Void and current_step_index = 0
			listeners_created: progress_listener /= Void
		end

feature {ANY}

	c_file, output_dir: STRING
	steps: ARRAY [ TRANSFORMATION_STEP ]
	current_step_index: INTEGER
	progress_listener: LIST [ PROCEDURE [ ANY, TUPLE [ INTEGER ] ] ]

	go (target_step_index: INTEGER) : BOOLEAN
		require
			target_step_index >= 1 and target_step_index <= steps.count
		local
			done, ok: BOOLEAN
		do
			ok := True
			from
				if target_step_index <= current_step_index then
					current_step_index := 1
				else -- target_step_index > current_step_index
					current_step_index := current_step_index + 1
				end
			until
				done
			loop
				notify_progress_listener
				ok := steps.item (current_step_index).process (Current)
				if not ok then
					done := True
					current_step_index := 0
					notify_progress_listener
				elseif current_step_index = target_step_index then
					done := True
				else
					current_step_index := current_step_index + 1
				end
			end
			Result := ok
		end

feature {NONE}

	notify_progress_listener
		do
			from
				progress_listener.start
			until
				progress_listener.after
			loop
				progress_listener.item.call ([ current_step_index ])
				progress_listener.forth
			end
		end

feature {ANY}

	initial_tree: ANTLR_AST_NODE assign set_initial_tree
	tree: AST_TRANSLATION_UNIT assign set_tree

	set_initial_tree (init_initial_tree: ANTLR_AST_NODE)
		do
			initial_tree := init_initial_tree
		end

	set_tree (init_tree: AST_TRANSLATION_UNIT)
		do
			tree := init_tree
		end

end
