class
	AST_C_CREATOR

inherit
	AST_DEFAULT_VISITOR
		redefine
			visit_translation_unit,
			visit_struct_or_union,
			visit_type_specifier,
			visit_declarator,
			visit_direct_declarator,
			visit_struct_declaration,
			visit_struct_declarator
		end
	ANTLR_AST_TYPE_CONSTANTS

create
	make

feature {NONE}

	make
		do
			create buffer.make_empty
		end

feature {ANY}

	buffer: STRING

feature {ANY}

	visit_translation_unit (node: AST_TRANSLATION_UNIT) : BOOLEAN
		do
			from
				node.declarations.start
			until
				node.declarations.after
			loop
				if not node.declarations.isfirst then
					buffer.append ("%N")
				end
				node.declarations.item.accept (Current)
				node.declarations.forth
			end
		end

	visit_struct_or_union (node: AST_STRUCT_OR_UNION) : BOOLEAN
		do
			if node.is_struct then
				buffer.append ("struct ")
			else
				buffer.append ("union ")
			end
			buffer.append (node.name)
			buffer.append (" {%N");
			indent := indent + 1
			from
				node.fields.start
			until
				node.fields.after
			loop
				print_indent
				node.fields.item.accept (Current)
				buffer.append (";%N")
				node.fields.forth
			end
			indent := indent - 1
			buffer.append ("};");
		end

	visit_type_specifier (node: AST_TYPE_SPECIFIER) : BOOLEAN
		do
			inspect
				node.type
			when C_kw_void then
				buffer.append ("void")
			when C_kw_char then
				buffer.append ("char")
			when C_kw_short then
				buffer.append ("short")
			when C_kw_int then
				buffer.append ("int")
			when C_kw_long then
				buffer.append ("long")
			when C_kw_float then
				buffer.append ("float")
			when C_kw_double then
				buffer.append ("double")
			when C_kw_signed then
				buffer.append ("signed")
			when C_kw_unsigned then
				buffer.append ("unsigned")
			when Art_type_id then
				buffer.append (node.identifier)
			when C_kw_struct then
				buffer.append ("struct ")
				buffer.append (node.identifier)
			when C_kw_union then
				buffer.append ("union ")
				buffer.append (node.identifier)
			end
		end

	visit_declarator (node: AST_DECLARATOR) : BOOLEAN
		do
			if node.direct_declarator /= Void then
				node.direct_declarator.accept (Current)
			end
		end

	visit_direct_declarator (node: AST_DIRECT_DECLARATOR) : BOOLEAN
		do
			if node.identifier /= Void then
				buffer.append (node.identifier)
			else
				buffer.append ("(")
				node.declarator.accept (Current)
				buffer.append (")")
			end
		end

	visit_struct_declaration (node: AST_STRUCT_DECLARATION) : BOOLEAN
		do
			from
				node.specifiers_and_qualifiers.start
			until
				node.specifiers_and_qualifiers.after
			loop
				node.specifiers_and_qualifiers.item.accept (Current)
				buffer.append (" ")
				node.specifiers_and_qualifiers.forth
			end
			from
				node.struct_declarators.start
			until
				node.struct_declarators.after
			loop
				node.struct_declarators.item.accept (Current)
				if not node.struct_declarators.islast then
					buffer.append (" ")
				end
				node.struct_declarators.forth
			end
		end

	visit_struct_declarator (node: AST_STRUCT_DECLARATOR) : BOOLEAN
		do
			if node.declarator /= Void then
				node.declarator.accept (Current)
			end
		end

feature {NONE} -- indent handling

	indent: INTEGER

	print_indent
		local
			i: INTEGER
		do
			from
				i := 1
			until
				i > indent
			loop
				buffer.append ("   ")
				i := i + 1
			end
		end

end
