class
	SOURCE_LOADER

inherit
	TRANSFORMATION_STEP
	ARGUMENTS

feature {ANY} -- TRANSFORMATION_STEP

	process (transformer: CODE_TRANSFORMER) : BOOLEAN
		local
			c_str: ANY
			antlr_c_root_node: POINTER
		do
			c_str := transformer.c_file.to_c
			antlr_c_root_node := create_tree ($c_str)
			if antlr_c_root_node /= default_pointer then
				transformer.initial_tree := create {ANTLR_AST_NODE}.make (antlr_c_root_node)
				Result := True
			else
				IO.put_string ("#################### [" + description + "] Parse Error: ####################%N")
				Result := False
			end
		end

	description : STRING
		do
			Result := "Loading Source File"
		end

feature {C2EIFFEL} -- c antlr interfacing

	create_tree (c_source_file: POINTER) : POINTER
		external
			"C inline use <CLexer.h>, <CParser.h>"
		alias
			"[
				pANTLR3_UINT8 fName;
				pANTLR3_INPUT_STREAM input;
				pCLexer lxr;
				pANTLR3_COMMON_TOKEN_STREAM tstream;
				pCParser psr;
				CParser_translation_unit_return langAST;

				fName = (pANTLR3_UINT8)$c_source_file;
				input = antlr3AsciiFileStreamNew(fName);
				if(input == NULL)
				{
					ANTLR3_FPRINTF(stderr, "Unable to open file %s due to malloc() failure\n", (char *)fName);
					exit(ANTLR3_ERR_NOMEM);
				}

				lxr = CLexerNew(input);
				if(lxr == NULL)
				{
					ANTLR3_FPRINTF(stderr, "Unable to create the lexer due to malloc() failure\n");
					exit(ANTLR3_ERR_NOMEM);
				}

				tstream = antlr3CommonTokenStreamSourceNew(ANTLR3_SIZE_HINT, lxr->pLexer->rec->state->tokSource);
				if(tstream == NULL)
				{
					ANTLR3_FPRINTF(stderr, "Out of memory trying to allocate token stream\n");
					exit(ANTLR3_ERR_NOMEM);
				}

				psr = CParserNew(tstream);
				if(psr == NULL)
				{
					ANTLR3_FPRINTF(stderr, "Out of memory trying to allocate parser\n");
					exit(ANTLR3_ERR_NOMEM);
				}

				langAST = psr->translation_unit(psr);
				return (psr->pParser->rec->state->errorCount == 0) ? langAST.tree : NULL;

				/* TODO: free the stuff when no longer needed
				// free langAST or langAST.tree?
				psr->free(psr);
				tstream->free(tstream);
				lxr->free(lxr);
				input->close(input);*/
			]"
		end

end
